# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 16.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V34
6992
7002
7104
7142
43472
43482
43520
43561
43600
43610
69734
69744
70480
70481
70494
70496
70528
70538
70539
70540
70542
70543
70544
70546
72016
72026
73440
73458
73552
73562
90368
90398
90416
90426
END
