+{
   locale_version => 1.31,
   entry => <<'ENTRY', # for DUCET v13.0.0
04D9      ; [.23C0.0020.0002] # CYRILLIC SMALL LETTER SCHWA
04D8      ; [.23C0.0020.0008] # CYRILLIC CAPITAL LETTER SCHWA
ENTRY
};
